﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Threading;

namespace dlytmusic.DataAccess {
    [System.ComponentModel.DesignerCategory("Code")] //<-- to not open in designer-mode. it does it since it it has some component ancestor
    public class BEWebClient : WebClient {
        private bool manual_unblock = false;
        private int timeout; //time in milliseconds
        public int Timeout {
            get { return timeout; }
            set { timeout = value; }
        }
        public BEWebClient(bool manualUnblock) {
            manual_unblock = manualUnblock;
            this.timeout = 60000;
            this.DownloadFileCompleted += new System.ComponentModel.AsyncCompletedEventHandler(BEWebClient_DownloadFileCompleted);
            this.DownloadDataCompleted += new System.Net.DownloadDataCompletedEventHandler(BEWebClient_DownloadDataCompleted);
            this.UploadDataCompleted += new System.Net.UploadDataCompletedEventHandler(BEWebClient_UploadDataCompleted);
        }
        public BEWebClient(int timeout = 60000) {
            this.timeout = timeout;
            manual_unblock = false;
            this.DownloadFileCompleted += new System.ComponentModel.AsyncCompletedEventHandler(BEWebClient_DownloadFileCompleted);
            this.DownloadDataCompleted += new System.Net.DownloadDataCompletedEventHandler(BEWebClient_DownloadDataCompleted);
            this.UploadDataCompleted += new System.Net.UploadDataCompletedEventHandler(BEWebClient_UploadDataCompleted);
        }

        EventWaitHandle asyncWait = new ManualResetEvent(false);
        //--
        public void DownloadFileWithEvents(string url, string outputPath) {
            asyncWait.Reset();
            this.DownloadFileAsync(new Uri(url), outputPath);
            asyncWait.WaitOne();
        }
        void BEWebClient_DownloadFileCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e) {
            if (!manual_unblock) asyncWait.Set();
        }
        //--
        public void DownloadDataWithEvents(string url) {
            asyncWait.Reset();
            this.DownloadDataAsync(new Uri(url));
            asyncWait.WaitOne();
        }
        void BEWebClient_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e) {
            if (!manual_unblock) asyncWait.Set();
            //reply would be ex:
            //byte[] data = (byte[])e.Result;
            //string reply = System.Text.Encoding.UTF8.GetString(data);
        }
        //--
        public void UploadDataWithEvents(string address, string method, byte[] data) {
            asyncWait.Reset();
            this.UploadDataAsync(new Uri(address), method, data);
            asyncWait.WaitOne();
        }
        void BEWebClient_UploadDataCompleted(object sender, UploadDataCompletedEventArgs e) {
            if (!manual_unblock) asyncWait.Set();
            //reply would be ex:
            //byte[] data = (byte[])e.Result;
            //string reply = System.Text.Encoding.UTF8.GetString(data);
        }

        public void UnblockIt() {
            asyncWait.Set();
        }


        //------------------------------------------------------
        protected override WebRequest GetWebRequest(Uri address) {
            var result = base.GetWebRequest(address);
            result.Timeout = this.timeout;
            return result;
        }

        private void InitializeComponent() {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(BEWebClient));
            this.Headers = ((System.Net.WebHeaderCollection)(resources.GetObject("$this.Headers")));
        }
    }
}

