﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Security.AccessControl;
using System.Security.Principal;

namespace dlytmusic.DataAccess {
    public class FileSys {

        public static bool hasWriteAccessToFolder(string folderPath) {
            try {
                System.Security.AccessControl.DirectorySecurity ds = Directory.GetAccessControl(folderPath); // This will raise an exception if the path is read only or do not have access to view the permissions. 
                return true;
            } catch (UnauthorizedAccessException) {
                return false;
            }
        }
        /// <summary>
        /// not used
        /// </summary>
        /// <param name="path"></param>
        /// <param name="accessRights"></param>
        /// <returns></returns>
        public static bool UserHasDirectoryAccessRights(string path, FileSystemRights accessRights) {
            var isInRoleWithAccess = false;

            try {
                var di = new DirectoryInfo(path);
                var acl = di.GetAccessControl();
                var rules = acl.GetAccessRules(true, true, typeof(NTAccount));

                var currentUser = WindowsIdentity.GetCurrent();
                var principal = new WindowsPrincipal(currentUser);
                foreach (AuthorizationRule rule in rules) {
                    var fsAccessRule = rule as FileSystemAccessRule;
                    if (fsAccessRule == null)
                        continue;

                    if ((fsAccessRule.FileSystemRights & accessRights) > 0) {
                        var ntAccount = rule.IdentityReference as NTAccount;
                        if (ntAccount == null)
                            continue;

                        if (principal.IsInRole(ntAccount.Value)) {
                            if (fsAccessRule.AccessControlType == AccessControlType.Deny)
                                return false;
                            isInRoleWithAccess = true;
                        }
                    }
                }
            } catch (UnauthorizedAccessException) {
                return false;
            }
            return isInRoleWithAccess;
        }

    }
}
