﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.IO;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Diagnostics;

namespace dlytmusic.Dbugns {
    public class Dbug {


        public static bool LOG_TO_FILE = false; // <--- set to false on real release release mode
        public static bool APPEND_LOG = true;


        public static void wline(string format, params object[] args) {
            System.Diagnostics.Debug.WriteLine(String.Format(format, args));
            writeLogIfOnLn(String.Format(format, args));
        }

        public static void wline(object arg) {
            System.Diagnostics.Debug.WriteLine(arg);
            writeLogIfOnLn(arg.ToString());
        }

        public static void wline(string txt) {
            System.Diagnostics.Debug.WriteLine(txt);
            writeLogIfOnLn(txt);
        }

        public static void w(string format, params object[] args) {
            System.Diagnostics.Debug.Write(String.Format(format, args));
            writeLogIfOn(String.Format(format, args));
        }

        public static void w(object arg) {
            System.Diagnostics.Debug.Write(arg);
            writeLogIfOn(arg.ToString());
        }

        public static void w(string txt) {
            System.Diagnostics.Debug.Write(txt);
            writeLogIfOn(txt);
        }

        #region logfile

        private static StreamWriter nonDbugStream;

        private static void writeLogIfOnLn(string txt) {
            if (LOG_TO_FILE) {
                if (nonDbugStream != null) {
                    try {
                        //nonDbugStream.WriteLine("{0}:{1}", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.ffffff"), txt);
                        nonDbugStream.WriteLine(txt);
                    } catch (Exception ex) {
                        Dbug.wline("Could not write to log file {0}, {1}, {2}", ex.GetType(), ex.Message, ex.StackTrace);
                    }
                }
            }
        }
        private static void writeLogIfOn(string txt) {
            if (LOG_TO_FILE) {
                if (nonDbugStream != null) {
                    try {
                        nonDbugStream.Write(txt);
                    } catch (Exception ex) {
                        Dbug.wline("Could not write to log file {0}, {1}, {2}", ex.GetType(), ex.Message, ex.StackTrace);
                    }
                }
            }
        }


        /// <summary>
        /// this is called once in App.xml.cs
        /// and called 3 times in program.cs (argument parsing mode on updates)
        /// </summary>
        public static void CloseDebugLog() {
            if (LOG_TO_FILE) {
                if (nonDbugStream != null) {
                    try {
                        nonDbugStream.WriteLine("-------------Log ends {0}------------", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.ffffff"));
                    } catch (Exception ex) {
                        Dbug.wline("Could not close log file {0}, {1}, {2}", ex.GetType(), ex.Message, ex.StackTrace);
                    } finally {
                        nonDbugStream.Close();
                    }
                }
            }
        }

        public static void OpenNonDebugLog() {
            if (LOG_TO_FILE) {
                try {

                    if (APPEND_LOG)
                        nonDbugStream = System.IO.File.AppendText(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "ReleaseModeOutput.log"));
                    else
                        nonDbugStream = System.IO.File.CreateText(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "ReleaseModeOutput.log"));

                    nonDbugStream.WriteLine("++++++++++++Log initiated {0}+++++++++++", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.ffffff"));
                    Version ver = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;
                    String _appInfoText = String.Format(
                        "Version: {0}.{1}\n" +
                        "Date: {2}\n"
                        , ver.Major
                        , ver.Minor
                        , new DateTime(2000, 1, 1, 12, 00, 00).AddDays(ver.Build).ToString(@"MM\/dd\/yyyy"));
                    nonDbugStream.WriteLine(_appInfoText);
                } catch (Exception ex) {
                    Dbug.wline("Could not open log file {0}, {1}, {2}", ex.GetType(), ex.Message, ex.StackTrace);
                    nonDbugStream = null;
                }
            }
        }

#if DEBUG
        public void addLogToFileDebugMode() {
            TextWriterTraceListener tr2 = new TextWriterTraceListener(
                APPEND_LOG ?
                System.IO.File.AppendText(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "DebugModeOutput.log"))
                :
                System.IO.File.CreateText(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "DebugModeOutput.log"))
            );
            Debug.Listeners.Add(tr2);
        }
#endif

        #endregion logfile

    };
}
