﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections;
using System.Windows;
using System.Collections.ObjectModel;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;
using System.Diagnostics;

using dlytmusic.Dbugns;
using dlytmusic.DataAccess;

using System.Windows.Forms;

namespace dlytmusic.ViewModel {

    public enum DownloadState { Waiting, Downloading, Converting, Done }

    public class DLitem : ViewModelBase, INotifyPropertyChanged {

        public DLitem(MainWindowVM aParent) {
            Parent = aParent;
            Prioridades = new ObservableCollection<string>() { "1", "2", "3" };
            Prioridade = string.IsNullOrWhiteSpace(Prioridade) ? "2" : Prioridade;

            State = DownloadState.Waiting;
            Progresso = 0;
            Title = "[Fetching title...]";
            pbIndeterminate = false; RaisePropertyChanged("pbIndeterminate");
            Result = null; RaisePropertyChanged("Result");
            //findTitle(); // não pode ser aqui por causa das inicializações em linha: Title, URL, etc
        }
        
        private MainWindowVM Parent;

        #region download end event ----++++
        public event NewWebTableRowEventHandler DownloadEndsEvt;
        #endregion download end event ----++++

        public string Title {get; set;}
        public string URL {get; set;}
        public string Filename { get; set; }
        public string Folder { get; set; }
        public string Qualidade { get; set; }
        public string Formato { get; set; }
        public int Progresso { get; set; }

        public bool pbIndeterminate { get; set; }
        public ObservableCollection<string> Prioridades { get; set; }
        public string Prioridade { get; set; }

        




        //-------------------------------------------------------------------------------------------

        public bool isBusy() {
            return State != DownloadState.Waiting && State != DownloadState.Done;
        }
        private DownloadState _State;
        public DownloadState State {
            get {
                return _State;
            }
            set {
                _State = value;
                Dbug.wline("state is {0}", value);
                switch (_State) {
                    case DownloadState.Waiting:
                        ButtonCancel_Visibility = Visibility.Visible;
                        ButtonStartNow_Visibility = Visibility.Visible;
                        ButtonOpenFolder_Visibility = Visibility.Collapsed;
                        ButtonClear_Visibility = Visibility.Collapsed;
                        break;
                    case DownloadState.Downloading:
                        ButtonCancel_Visibility = Visibility.Visible;
                        ButtonStartNow_Visibility = Visibility.Collapsed;
                        ButtonOpenFolder_Visibility = Visibility.Collapsed;
                        ButtonClear_Visibility = Visibility.Collapsed;
                        break;
                    case DownloadState.Converting:
                        ButtonCancel_Visibility = Visibility.Visible;
                        ButtonStartNow_Visibility = Visibility.Collapsed;
                        ButtonOpenFolder_Visibility = Visibility.Collapsed;
                        ButtonClear_Visibility = Visibility.Collapsed;
                        break;
                    case DownloadState.Done:
                        ButtonCancel_Visibility = Visibility.Collapsed;
                        ButtonStartNow_Visibility = Visibility.Collapsed;
                        ButtonOpenFolder_Visibility = Visibility.Visible;
                        ButtonClear_Visibility = Visibility.Visible;
                        break;
                }
                RaisePropertyChanged("ButtonCancel_Visibility");
                RaisePropertyChanged("ButtonStartNow_Visibility");
                RaisePropertyChanged("ButtonOpenFolder_Visibility");
                RaisePropertyChanged("ButtonClear_Visibility");
            }
        }

        public Visibility ButtonCancel_Visibility { get; set; }
        public Visibility ButtonStartNow_Visibility { get; set; }
        public Visibility ButtonOpenFolder_Visibility { get; set; }
        public Visibility ButtonClear_Visibility { get; set; }

        public FileInfo Result { get; set; }


        public RelayCommand ButtonStartNowCmd {
            get {
                return new RelayCommand(cmdparam => {
                    Dbug.wline("------ButtonStartNowCmd------");
                    startDLOperation();
                });
            }
        }

        public RelayCommand ButtonCancelCmd {
            get {
                return new RelayCommand(cmdparam => {
                    Dbug.wline("-----ButtonCancelCmd-------");
                    if (backworker != null) {
                        if (backworker.IsBusy && !backworker.CancellationPending) {
                            backworker.CancelAsync();
                        }
                    }
                    Parent.removeDlitem(this);
                });
            }
        }

        public RelayCommand ButtonOpenFolderCmd {
            get {
                return new RelayCommand(cmdparam => {
                    Dbug.wline("------ButtonOpenFolderCmd------");
                    //C:\Windows\explorer.exe /n, /e, /select,  "C:\Progdev"
                    string expath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Windows), "explorer.exe");
                    if (Result != null) {
                        System.Diagnostics.Process.Start(expath, "/n, /e, /select,  \"" + Result.FullName + "\"");
                    } else {
                        //System.Windows.MessageBox.Show("An error occurred in the download");
                        askErrUpdateYoutube_dl();
                    }
                    Parent.removeDlitem(this);
                });
            }
        }

        private void askErrUpdateYoutube_dl(){
            MessageBoxResult resul = System.Windows.MessageBox.Show("An error occurred in the download. Update youtube-dl.exe?", "Update", System.Windows.MessageBoxButton.YesNo);
            if (resul == MessageBoxResult.Yes) {
                string strCmd = "/C youtube-dl.exe -U";
                System.Diagnostics.Process.Start("CMD.exe", strCmd);
            }
        }


        public RelayCommand ButtonClearCmd {
            get {
                return new RelayCommand(cmdparam => {
                    Dbug.wline("------ButtonClearCmd------");
                    Parent.removeDlitem(this);
                });
            }
        }




        private BackgroundWorker backworker = null;
        public void startDLOperation() {
            using (backworker = new BackgroundWorker() { WorkerReportsProgress = true, WorkerSupportsCancellation = true }) {

                backworker.DoWork += (s, a) => {
                    BackgroundWorker self_bgw = s as BackgroundWorker;
                    ArrayList argsList = a.Argument as ArrayList;
                    string ytcode = argsList[0] as string;
                    string aformat = argsList[1] as string;
                    string aquality = argsList[2] as string;
                    string filename = argsList[3] as string;
                    string folder = argsList[4] as string;

                    FileInfo mp3_fileInfo = null;

                    string currentProcPath = System.Reflection.Assembly.GetEntryAssembly().Location;
                    FileInfo fileExe = new FileInfo(currentProcPath);
                    string newProcessFullName = Path.Combine(fileExe.DirectoryName, "youtube-dl.exe");
                    //string newProcessArgs = "--max-quality mp4 http://www.youtube.com/watch?v=DbAFfKaHBAE -x --audio-format mp3 --audio-quality 224k --restrict-filenames";
                    string newProcessArgs = " --max-quality mp4" +
                                            " http://www.youtube.com/watch?v=" + ytcode +
                                            " -x" +
                                            " --audio-format " + aformat +
                                            " --audio-quality " + aquality +
                                            " --restrict-filenames" +
                                            " -o \"" + folder + "\\" + filename + ".%(ext)s\"";
                    Dbug.wline("args: {0}", newProcessArgs);
                    ProcessStartInfo startInfo = new ProcessStartInfo(newProcessFullName, newProcessArgs);
                    startInfo.WindowStyle = ProcessWindowStyle.Hidden;
                    startInfo.CreateNoWindow = true;
                    startInfo.UseShellExecute = false;
                    startInfo.RedirectStandardOutput = true;

                    using (Process process = Process.Start(startInfo)) {
                        using (StreamReader reader = process.StandardOutput) {
                            
                            while (!reader.EndOfStream && !self_bgw.CancellationPending) {
                                string line = reader.ReadLine();
                                Dbug.wline(line);//[download]  53.5% of 34.03M at  313.17k/s ETA 00:51
                                string[] words = line.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                                if (words.Length >= 2) {
                                    string firstw = words[0];
                                    string secondw = words[1];
                                    switch (firstw) {
                                        case "[youtube]":
                                            self_bgw.ReportProgress(0, firstw);
                                            break;
                                        case "[download]":
                                            if (secondw[secondw.Length - 1] == '%') {
                                                string pcs = secondw.Substring(0, secondw.Length - 3);//xx.5%
                                                int pc=0;
                                                try {
                                                    pc = int.Parse(pcs);
                                                } catch (Exception) {
                                                    Dbug.wline("Could not read int");
                                                }
                                                self_bgw.ReportProgress(pc, firstw);
                                            }
                                            break;
                                        case "[ffmpeg]": //[ffmpeg] Destination: C:\Users\lau\Music\Festergut_Kill_-_A_Guild_dos_Tugas.wmv-DbAFfKaHBAE.mp3
                                            self_bgw.ReportProgress(100, firstw);
                                            if (secondw == "Destination:") {
                                                mp3_fileInfo = new FileInfo(line.Substring(22));
                                            }
                                            break;
                                        case "Deleting":
                                            self_bgw.ReportProgress(100, firstw);
                                            break;
                                        default://[??] 
                                            self_bgw.ReportProgress(-1, firstw);
                                            break;
                                    }
                                }
                            }
                            if (self_bgw.CancellationPending) {
                                process.Kill();
                                process.Close();
                                //process.CloseMainWindow();
                                //process.StandardInput.Write("\x3");
                                //Thread.Sleep(500);
                                //process.StandardInput.Write("y\r\n");
                            }
                        }
                    }

                    if (self_bgw.CancellationPending) {
                        a.Cancel = true;
                    } else {
                        self_bgw.ReportProgress(100, "");
                        Thread.Sleep(10);
                        a.Result = mp3_fileInfo;
                    }

                };
                backworker.ProgressChanged += (s, a) => {
                    Progresso = a.ProgressPercentage; RaisePropertyChanged("Progresso");
                    string dlst = (string)a.UserState;
                    if (State == DownloadState.Downloading && dlst == "[ffmpeg]") {
                        State = DownloadState.Converting; RaisePropertyChanged("State");
                        pbIndeterminate = true; RaisePropertyChanged("pbIndeterminate");
                    }
                };
                backworker.RunWorkerCompleted += (s, a) => {
                    if (!a.Cancelled && a.Error == null) {
                        State = DownloadState.Done; RaisePropertyChanged("State");
                        DownloadEndsEvt(this, new DownloadEndsEventArgs() { wascanceled = false, waserror = false });
                        Result = (FileInfo)a.Result; RaisePropertyChanged("Result");

                        if (Result == null) {
                            askErrUpdateYoutube_dl();
                        }

                    } else {
                        DownloadEndsEvt(this, new DownloadEndsEventArgs() { wascanceled = a.Cancelled, waserror = a.Error != null });
                    }
                    pbIndeterminate = false; RaisePropertyChanged("pbIndeterminate");
                };
                State = DownloadState.Downloading; RaisePropertyChanged("State");
                backworker.RunWorkerAsync(
                    new ArrayList() {
                        SetupItem.getCode(URL),
                        Formato,
                        Qualidade,
                        Filename,
                        Folder
                    }
                );
            }

        }


        public void findTitle() {
            using (BackgroundWorker bgw = new BackgroundWorker() { WorkerReportsProgress = true, WorkerSupportsCancellation = true }) {
                bgw.DoWork += (s, a) => {
                    BackgroundWorker self_bgw = s as BackgroundWorker;
                    //ArrayList argsList = a.Argument as ArrayList;
                    //DLitem dli = argsList[0] as DLitem;
                    //string requestURL = dli.URL;
                    string requestURL = a.Argument as string;
                    string reply = "";
                    bool gotreply = false;
                    try {
                        using (BEWebClient webclient = new BEWebClient()) {
                            webclient.Headers.Add(HttpRequestHeader.Accept, "text/plain");
                            webclient.Headers.Add(HttpRequestHeader.ContentType, "application/x-www-form-urlencoded");
                            webclient.Headers.Add(HttpRequestHeader.UserAgent, Properties.Resources.HttpUserAgentToUse);
                            webclient.DownloadDataCompleted += (complsender, ddea) => {
                                try {
                                    reply = System.Text.Encoding.UTF8.GetString(ddea.Result);
                                    gotreply = true;
                                    //Dbug.wline("REPLY->Requested {0}\n\tand got Reply: {1}", requestURL, reply);
                                } catch (Exception ex) {
                                    Dbug.wline("EXCEPT->findTitle: Requested {0}\n\tand got exception {1} {2} {3}", requestURL, ex.GetType(), ex.Message, ex.StackTrace);
                                }
                            };
                            Dbug.wline("Will request {0}", requestURL);
                            webclient.DownloadDataWithEvents(requestURL);
                            for (int i = 0; i < 10 && !gotreply; i++) {
                                Thread.Sleep(100);//1 sec para dar tempo para fill reply
                            }
                        }
                    } catch (Exception ex) {
                        Dbug.wline("Could not findTitle: {0}: {1}: {2}", ex.GetType(), ex.Message, ex.StackTrace);
                    }

                    Regex regTitlefield = new Regex("<title>(?<titulo>.+)\\s-\\sYouTube</title>");
                    // <title>(?<titulo>.+)\s-\sYouTube</title>
                    // \stitle=\"(?<titulo>[^\"]+)\"  ------ "\\stitle=\\\"(?<titulo>[^\\\"]+)\\\""
                    string titulo = "";
                    if (regTitlefield.IsMatch(reply, 0)) {
                        MatchCollection mc = regTitlefield.Matches(reply);
                        titulo = mc[0].Groups["titulo"].Value;
                    }
                    Dbug.wline("Got titulo {0}",titulo);
                    a.Result = titulo;
                };
                bgw.ProgressChanged += (s, a) => {
                    Dbug.wline("findTitle {0} %", (int)a.ProgressPercentage);
                };
                bgw.RunWorkerCompleted += (s, a) => {
                    if (!a.Cancelled) {
                        Title = a.Result as string;
                        RaisePropertyChanged("Title");
                    }
                };
                //bgw.RunWorkerAsync(new ArrayList() { this });
                Dbug.wline("Sending to title worker {0}",this.URL);
                bgw.RunWorkerAsync(this.URL);
            }




        }

        #region INotifyPropertyChanged
        public new event PropertyChangedEventHandler PropertyChanged; //overridden base class with new
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e) {
            if (PropertyChanged != null) {
                PropertyChanged(this, e);
                //Dbug.wline("raised " + e.PropertyName);
            }
        }

        protected void RaisePropertyChanged(string propertyName) {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        public void RaisePropertyChangedP(string propertyName) {
            RaisePropertyChanged(propertyName);
        }

        #endregion
    }

    //----------------------------------------------------------------------------------------
    #region download end event ----++++
    public class DownloadEndsEventArgs : EventArgs {
        public bool wascanceled { get; set; }
        public bool waserror { get; set; }
    }
    public delegate void NewWebTableRowEventHandler(DLitem src, DownloadEndsEventArgs ea);
    #endregion download end event ----++++
    //----------------------------------------------------------------------------------------

}
