﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.ComponentModel;
using dlytmusic.Dbugns;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
//using Microsoft.Win32;
using System.Windows.Forms;

namespace dlytmusic.ViewModel {
    public class SetupItem : ViewModelBase, INotifyPropertyChanged {

        public string _URL;
        public string URL { 
            get { return _URL; } 
            set { Properties.Settings.Default.Setup_URL = _URL = value; } 
        }

        public string _Filename;
        public string Filename { 
            get { return _Filename; }
            set { 
                Properties.Settings.Default.Setup_Filename = _Filename = value;
                //Dbug.wline("file name set");
            } 
        }

        public string _Folder;
        public string Folder { 
            get { return _Folder; }
            set { Properties.Settings.Default.Setup_Folder = _Folder = value; } 
        }

        public ObservableCollection<string> Qualidades { get; set; }
        public string _Qualidade;
        public string Qualidade {
            get { return _Qualidade; }
            set { Properties.Settings.Default.Setup_Qualidade = _Qualidade = value; }
        }

        public ObservableCollection<string> Formatos { get; set; }
        public string _Formato;
        public string Formato {
            get { return _Formato; }
            set { Properties.Settings.Default.Setup_Formato = _Formato = value; } 
        }

        public ObservableCollection<string> Prioridades { get; set; }
        public string _Prioridade;
        public string Prioridade {
            get { return _Prioridade; }
            set { Properties.Settings.Default.Setup_Prioridade = _Prioridade = value; }
        }


        public SetupItem() {
            URL = Properties.Settings.Default.Setup_URL;
            Filename = Properties.Settings.Default.Setup_Filename;
            Folder = Properties.Settings.Default.Setup_Folder;

            Formatos = new ObservableCollection<string>() { "best", "aac", "vorbis", "mp3", "m4a", "opus", "wav" };
            Formato = Properties.Settings.Default.Setup_Formato;
            Formato = string.IsNullOrWhiteSpace(Formato) ? "mp3" : Formato;

            Prioridades = new ObservableCollection<string>() { "1", "2", "3" };
            Prioridade = Properties.Settings.Default.Setup_Prioridade;
            Prioridade = string.IsNullOrWhiteSpace(Prioridade) ? "2" : Prioridade;

            //32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256 and 320 kbit/s
            Qualidades = new ObservableCollection<string>() { "32K", "40K", "48K", "56K", "64K", "80K", "96K", "112K", "128K", "160K", "192K", "224K", "256K", "320K" };
            //Qualidades = new ObservableCollection<string>() { "0", "1", "2","3","4","5","6","7","8","9" };
            Qualidade = Properties.Settings.Default.Setup_Qualidade;
            Qualidade = string.IsNullOrWhiteSpace(Qualidade) ? "160K" : Qualidade;
            
        }

        public RelayCommand ChangeFolder {
            get {
                return new RelayCommand(cmdparam => {
                    //Dbug.wline("Command ({0}) text is {1}", cmdparam, Organization);
                    //System.Windows.MessageBox.Show("ChangeFolder");
                    changeFolderDialog();
                });
            }
        }

        private void changeFolderDialog() {
            using (FolderBrowserDialog dlg = new FolderBrowserDialog()) {
                dlg.Description = "Select a folder to save the file in";
                dlg.SelectedPath = getValidMusicFolder(Folder);
                dlg.ShowNewFolderButton = true;
                DialogResult result = dlg.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK) {
                    Folder = dlg.SelectedPath;
                    RaisePropertyChanged("Folder");
                }
            }
        }


        public void validateSetup() {
            Folder = getValidMusicFolder(Folder);

            if (string.IsNullOrWhiteSpace(Filename) || Filename.IndexOfAny(Path.GetInvalidFileNameChars()) >= 0) {
                //File.Exists(Path.Combine(someFolder, filename)
                Filename = "%(title)s-%(id)s";
            }
            URL = trimUrl(URL);
        }

        public static string getValidMusicFolder(string folderpath) {
            string retval = folderpath;
            if (string.IsNullOrWhiteSpace(folderpath) || folderpath.IndexOfAny(Path.GetInvalidPathChars()) < 0) {
                if (!Directory.Exists(folderpath)) {
                    retval = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
                }
            } else {
                retval = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            }
            return retval;
        }
        public static string trimUrl(string s) {
            Regex cfield = new Regex(@"^(?:[hH][tT][tT][pP][sS]?:\/\/)?(?:(?:[wW][wW][wW]\.[yY][oO][uU][tT][uU][bB][eE]\.[cC][oO][mM]\/)).+[&\?]v=(?<code>.{11})");
            string retval = "https://www.youtube.com/watch?v=";
            if (cfield.IsMatch(s, 0)) {
                MatchCollection mc = cfield.Matches(s);
                retval += mc[0].Groups["code"].Value;
            }
            return retval;
        }
        public static string getCode(string s) {
            Regex cfield = new Regex(@"^(?:[hH][tT][tT][pP][sS]?:\/\/)?(?:(?:[wW][wW][wW]\.[yY][oO][uU][tT][uU][bB][eE]\.[cC][oO][mM]\/)).+[&\?]v=(?<code>.{11})");
            string retval = null;
            if (cfield.IsMatch(s, 0)) {
                MatchCollection mc = cfield.Matches(s);
                retval = mc[0].Groups["code"].Value;
            }
            return retval;
        }
        public static bool okUrl(string s) {
            Regex cfield = new Regex(@"^(?:[hH][tT][tT][pP][sS]?:\/\/)?(?:(?:[wW][wW][wW]\.[yY][oO][uU][tT][uU][bB][eE]\.[cC][oO][mM]\/)).+[&\?]v=(?<code>.{11})");
            return cfield.IsMatch(s, 0);
        }


        public string toString() {
            return string.Format("URL: {0}, Filename: {1}, \nFolder: {2}, \nQualidade: {3}, Formato: {4}, Prioridade: {5}",
                URL, Filename, Folder, Qualidade, Formato, Prioridade);
        }





        //-------------------------------------------------------------------------------------------

        #region INotifyPropertyChanged
        public new event PropertyChangedEventHandler PropertyChanged; //overridden base class with new
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e) {
            if (PropertyChanged != null) {
                PropertyChanged(this, e);
                Dbug.wline("raised " + e.PropertyName);
            }
        }

        protected void RaisePropertyChanged(string propertyName) {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        public void RaisePropertyChangedP(string propertyName) {
            RaisePropertyChanged(propertyName);
        }

        #endregion



    }
}
